#Copyright 2008 Alexander Exner

#This file is part of WebcamDemo.

#WebcamDemo is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#WebcamDemo is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with WebcamDemo.  If not, see <http://www.gnu.org/licenses/>.

import numpy
import Image

def imageToArray(i):
    #a = numpy.array(i.tostring(), dtype = numpy.int8)
    a = numpy.array(list(i.getdata()), dtype = numpy.uint8)
    a.shape = i.size[1], i.size[0]
    return a

def arrayToImage(a):
    return Image.fromstring("L", (a.shape[1], a.shape[0]), a.tostring())

def toDebugString(a):
    #numpy.array2string(a, precision=2, separator=',', suppress_small = True)
    return numpy.array2string(a, precision=2)

def getVector(a):
    """
    Returns a Vector (percentage 0 - 100).
    This Vector has direction and size of maximum values in the matrix (array)
    This Function is slow
    """
    (sizey, sizex) = a.shape
    ux = sizex / 2
    uy = sizey / 2
    vmax = (numpy.int32)(255 * sizex * sizey / 2)
    x = (numpy.int32)(0)
    y = (numpy.int32)(0)
    for i in range(sizex):
        for j in range(sizey):
            value = a[j, i]
            if i <= ux:
                x -= value
            else:
                x += value
            if j <= uy:
                y -= value
            else:
                y += value
    return (x * 100 / vmax , y * 100 / vmax)

def getWeightMatrix(r):
    """
    Returns a (r*2+1) x (r*2+1) matrix M.
    M(r+1, r+1) has value 1.
    The value of all other elements decrease with euclidean distance
    This Function is slow
    """
    d = r * 2 + 1
    a = numpy.zeros(d * d).reshape(d, d)
    for i in range(d):
        for j in range(d):
            euclidean = numpy.sqrt((i - r) ** 2 + (j - r) ** 2)
            tmp = euclidean / r
            if tmp > 1.0:
                tmp = 1.0
            a[i, j] = 1 - tmp
    return a

def RsubGB(r, g, b):
    z = numpy.zeros_like(r)
    gb = (g + b) / 2
    for i in range(r.shape[0]):
        for j in range(r.shape[1]):
            red = r[i, j]
            other = g[i, j] / 2 + b[i, j] / 2
            diff = red - other
            if diff > 0 and diff < 128:
                z[i, j] = (numpy.uint8)(diff * 2)
            if diff >= 128:
                z[i, j] = (numpy.uint8)(255)
    return z

